//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  installmgr.cpp: module installation manager
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "installmgr.h"
#include "interface.h"
#include "support.h"
#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>

#include "dkmsinstallmgr.h"

#include <iostream>
using std::cout;
using std::endl;

using namespace gmodcfg;

InstallManager* InstallManager::TheInstance = NULL;

//--------------------------------------------------------------------------//
//  Returns the only instance of the install manager
//--------------------------------------------------------------------------//
void
InstallManager::Create( GtkWidget* appWidget, int verbosity )
{
    if ( !TheInstance )
        TheInstance = new DkmsInstallManager( appWidget, verbosity );
}

InstallManager*
InstallManager::GetInstallManager()
{
    return TheInstance;
}

//--------------------------------------------------------------------------//
//  C'tor
//--------------------------------------------------------------------------//
InstallManager::InstallManager( GtkWidget* appWidget, int verbosity )
    : InstallWindow(0), AppWidget(appWidget), Verbosity(verbosity),
      PathEntry(0), StatusBar(0)
{
}

//--------------------------------------------------------------------------//
//  Helper function to update the progress and status bar
//--------------------------------------------------------------------------//
void
InstallManager::UpdateProgressBar( gfloat percent ) const
{
    gnome_appbar_set_progress_percentage( StatusBar, percent );
}

void
InstallManager::WriteStatusBarMessage( const char* msg ) const
{
    gnome_appbar_set_status( StatusBar, msg );
}

//--------------------------------------------------------------------------//
//  Progress callback: updates the progress bar
//--------------------------------------------------------------------------//
static gint
progressCallback( GnomeVFSXferProgressInfo* info, gpointer data )
{
    GnomeVFSFileSize fileSize = info->file_size;
    GnomeVFSFileSize copied = info->bytes_copied;
    InstallManager* mgr = static_cast<InstallManager*>(data);

    if ( fileSize )
    {
        gfloat progress = (gfloat)copied / (gfloat)fileSize;
        mgr->UpdateProgressBar( progress );

        while ( gtk_events_pending() )
            gtk_main_iteration(); // used to refresh GUI
    }
    return TRUE;
}

//--------------------------------------------------------------------------//
//  Download a file pointed by "src", save it as "dest".
//  Updates the progress bar as download progresses.
//--------------------------------------------------------------------------//
bool
InstallManager::downloadFile( const gchar* src, const gchar* dest ) const
{
    if ( Verbosity >= 3 )
        cout << "Downloading file from " << src << endl;

    WriteStatusBarMessage( "" );

    GnomeVFSURI* srcUri = gnome_vfs_uri_new( src );
    GnomeVFSURI* fileUri = gnome_vfs_uri_new( "file://" );
    GnomeVFSURI* destUri = gnome_vfs_uri_append_string( fileUri, dest );

    // If we're downloading something, say so in the status bar
    const gchar* scheme = gnome_vfs_uri_get_scheme( srcUri );
    bool notifyUser = ( strcmp(scheme, "file") != 0 );

    // TODO: don't call vfs when src is a local file
    if ( notifyUser )
        WriteStatusBarMessage( _("Downloading package...") );

    GnomeVFSResult result = gnome_vfs_xfer_uri( srcUri, destUri,
                            GNOME_VFS_XFER_DEFAULT,
                            GNOME_VFS_XFER_ERROR_MODE_ABORT,
                            GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
                            progressCallback, (gpointer)this );

    gnome_vfs_uri_unref( destUri );
    gnome_vfs_uri_unref( fileUri );
    gnome_vfs_uri_unref( srcUri );

    if ( result != GNOME_VFS_OK )
    {
        gchar* msg = g_strdup_printf( _("Error: %s"), 
                                      gnome_vfs_result_to_string( result ) );
        WriteStatusBarMessage( msg );
        g_free( msg );
        return false;
    }

    if ( notifyUser )
        WriteStatusBarMessage( _("Downloading is complete.") );

    return true;
}

//--------------------------------------------------------------------------//
//  Bring up install window
//--------------------------------------------------------------------------//
void
InstallManager::StartInstall( const string& path )
{
    InstallWindow = create_install();
    gtk_window_set_transient_for( GTK_WINDOW(InstallWindow),
                                  GTK_WINDOW(AppWidget) );
    gtk_window_present( GTK_WINDOW(InstallWindow) );
    PathEntry = GTK_ENTRY( lookup_widget( InstallWindow,
                                          "install_package_combo_entry") );
    StatusBar = GNOME_APPBAR(lookup_widget( InstallWindow, "install_appbar" ));

    // Populate the entry widget if path is provided
    if ( !path.empty() )
    {
        gtk_entry_set_text( PathEntry, path.c_str() );
        if ( Verbosity >= 3 )
            cout << "path = " << path << endl;
    }
}

//--------------------------------------------------------------------------//
//  Proceed with installation (OK button callback)
//--------------------------------------------------------------------------//
void
InstallManager::DoInstall()
{
    UpdateProgressBar( 0 );
    const gchar* path = gtk_entry_get_text( PathEntry );
    if ( !path || !strcmp(path, "") )
        return;

    char tmpFile[] = "/tmp/gmodconfigXXXXXX";
    if ( mkstemp( tmpFile ) == -1 )
    {
        cout << "Could not create temp file" << endl;
        return;
    }

    // Download package file to /tmp and install it
    if ( downloadFile( path, tmpFile ) && installPackage( tmpFile ) )
        WriteStatusBarMessage( _("Installation succeeded") );

    if ( Verbosity < 2 ) // keep temp file around for debug purpose
        unlink( tmpFile );

    UpdateProgressBar( 0 );
}

//--------------------------------------------------------------------------//
//  Cancel installation (Cancel button callback)
//--------------------------------------------------------------------------//
void
InstallManager::CancelInstall()
{
    gtk_widget_destroy( InstallWindow );
}

//--------------------------------------------------------------------------//
//  Bring up uninstall window
//--------------------------------------------------------------------------//
void
InstallManager::StartUninstall()
{
    // TODO
}

//--------------------------------------------------------------------------//
//  Proceed with uninstallation (OK button callback)
//--------------------------------------------------------------------------//
void
InstallManager::DoUninstall()
{
    // TODO
}

//--------------------------------------------------------------------------//
//  Cancel uninstallation (Cancel button callback)
//--------------------------------------------------------------------------//
void
InstallManager::CancelUninstall()
{
    // TODO
}

//--------------------------------------------------------------------------//
//  Cancel uninstallation (Cancel button callback)
//--------------------------------------------------------------------------//
bool
InstallManager::IsPackageInstalled( const string& pkgName ) const
{
    return isPackageInstalled( pkgName );
}

